# Porting an AMS sensor driver and operation in the Linux kernel.
version 1.13


## Description

  - `arch/arm/boot/dts/`
    This contains the device tree overlay file that is an example of how the
    sensor was added to a Raspberry Pi BSP for development/testing.

  - `driver/`
    This directory contains the source and includes to the device driver and
    was built and tested against linux kernel 4.9 on a Raspberry Pi 3B

  - `include/`
    This directory contains the device specific include files that are needed
    by both the board config source and the driver source.



## Cross-Compiling

  Follow these steps, then the subsection for the component you are compiling -

  1. Download Raspberry Pi Cross Compile Toolchain:
     ```
     git clone git://github.com/raspberrypi/tools.git --depth=1
     ```

  2. Set `RPI_TOOLCHAIN` environment variable to point to the toolchain downloaded in Step 1.

  3. Download Raspberry Pi Linux Kernel source:
     1. `git clone --branch <kernel_version_branch> --depth=1 https://github.com/raspberrypi/linux`
     2. Driver tested against **Kernel version 4.9.59-v7+**, tag=**raspberrypi-kernel_1.20171029-1**

  4. Set `LINUX_SRC` environment variable to point to the kernel source downloaded in Step 3.

  5. A convenience script `./rpi.env` is included to setup Cross Compilation environment
     variables before compiling
     `source ./rpi.env`   **requires RPI_TOOLCHAIN and LINUX_SRC env variables set**


  6. Compile Raspberry Pi Linux Kernel
     1. `make bcm2709_defconfig`
     2. `make modules_prepare`
     3. `make -j 4 zImage modules dtbs`
        This can take a while, as it is compiling the whole kernel. This
        step is necessary to create the Module.symvers needed to build
        external modules
     4. Reference Material for Raspberry Pi Cross compiling:
        - [Official Raspberry Pi documentation for building the kernel][2]
        - [Other Raspberry Pi Kernel compilation resources][1]

  7. Compile Device Driver:
     1. `source ./rpi.env` to setup for cross compiling
     2. `cd ./driver`
     3. `make clean`
     4. `make CONFIG_SENSORS_TMD2755=m`
     5. copy tmd2755.ko to Raspberry Pi

  8. Compile Device Tree overlay
     1. `cd ./arch/arm/boot/dts`
     2. `dtc -@ -I dts -O dtb -o tmd2755-overlay.dtbo tmd2755-overlay.dts`
     3. `cp tmd2755-overlay.dtbo  /boot/overlays/`  (on the Raspberry Pi)
     4. add `dtoverlay=tmd2755-overlay` to end of text file `/boot/config.txt`
     5. `sync` to make sure all cahced writes are flushed (requires sudo)
     6. Raspberry Pi must be rebooted to take effect - `sudo shutdown -r now`



## Operation Guide

  1. In the Raspberry Pi shell, become root with:
     ```
     sudo su
     ```

  2. Transfer the driver module to the raspberry pi (if it is not already in the image).

  3. Inserting/Removing the driver module
     ```
     insmod /home/pi/example.ko
     rmmod example
     ```

  4. Check the input device names in sysfs to find to find the features supported
     with this sensor.
     ```
     root@raspberrypi:~# cat /sys/class/input/event0/device/name
     tmd2755-prox
     root@raspberrypi:~# cat /sys/class/input/event1/device/name
     tmd2755-als
     ```

     *Note: if there are other input devices on system(e.g. keyboard/mouse), the event ID might not be 0 and 1.*

  5. Read from the input device files to get sensor data, event ID depends on what you got in step 4:
     ```
     od -l /dev/input/eventX
     ```



## ABI descriptions

  Change directory to path of desired input device sysfs directory
  ```
  cd /sys/class/input/inputX/
  ```

  Proximity sysfs entries | Description
  :---                    | :---
  ./prx_detect            | read the current proximity detection event state
  ./prx_raw               | read the raw proximity adc count
  ./prx_regs              | read proximity config registers
  ./prx_persist           | read/write the proximity persistance settings
  ./prx_offset            | read/write the proximity offset
  ./prx_gain              | read/write the proximity gain value
  ./prx_enable            | read/write proximity enabled (automatically enabled when the input device is opened)


  ALS sysfs entries       | Description
  :---                    | :---
  ./als_atime             | read/write the ALS integration time
  ./als_auto_gain         | read/write the ALS autogain feature
  ./als_cpl               | read the current CPL value
  ./als_gain              | read/write the gain setting
  ./als_lux               | read the calculated lux value
  ./als_lux_coef          | read/write the lux equation coefficients
  ./als_persist           | read/write the ALS persistance settings
  ./als_enable            | read/write ALS enabled (auto enabled when the input device is opened)
  ./als_thresh_deltaP     | read/write ALS deltaP setting
  ./als_wtime             | read/write the wait time before ALS readings



## Hardware Wiring to Raspberry Pi 3b

  This diagram is for wiring an Official AMS 7-pin evaluation Daughter card.

  [Raspberry Pi 3b 40-pin connector reference][3]


    Raspberry Pi 3b:                                 AMS 7-pin Daughter card:

                    -----                                              --
    Pin_1    3.3V - |* *|                                         X --| Pin_1
          I2C_SDA - |* *|                                       VDD --|
          I2C_SCL - |* *| - GND                                 GND --|
                    |* *|                                   I2C_SDA --|
                    |* *|                                   I2C_SCL --|
                    |* *|                                       INT --|
                    |* *|                                         X --|
                    |* *|                                              --
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *|
                    |* *| - GPIO20 (INT)
                    |* *|
                    -----

[1]: http://elinux.org/Raspberry_Pi_Kernel_Compilation
[2]: https://www.raspberrypi.org/documentation/linux/kernel/building.md
[3]: https://pinout.xyz/
