/*
 *****************************************************************************
 * Copyright by ams AG                                                       *
 * All rights are reserved.                                                  *
 *                                                                           *
 * IMPORTANT - PLEASE READ CAREFULLY BEFORE COPYING, INSTALLING OR USING     *
 * THE SOFTWARE.                                                             *
 *                                                                           *
 * THIS SOFTWARE IS PROVIDED FOR USE ONLY IN CONJUNCTION WITH AMS PRODUCTS.  *
 * USE OF THE SOFTWARE IN CONJUNCTION WITH NON-AMS-PRODUCTS IS EXPLICITLY    *
 * EXCLUDED.                                                                 *
 *                                                                           *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS         *
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT  *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE     *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.      *
 *****************************************************************************
 */

#ifndef __AMS_TMD2755_THERMAL_DRIFT_H__
#define __AMS_TMD2755_THERMAL_DRIFT_H__

/* Dift table */
#define DRIFT_TABLE_SIZE             (11)
#define DRIFT_TABLE_DEFAULT_VAL    (-50)

/* The are ADC counts for the temp sensor */
/* As temp increases, counts decrease */

#define DRIFT_TEMP_BIN_0_LOWER    (411)       /* < -5 degrees  */
#define DRIFT_TEMP_BIN_0_UPPER    (600)       /* Not used - placeholder       */
#define DRIFT_TEMP_BIN_1_LOWER    (395)                      /* -5 to 5       */
#define DRIFT_TEMP_BIN_1_UPPER    (DRIFT_TEMP_BIN_0_LOWER)
#define DRIFT_TEMP_BIN_2_LOWER    (382)                      /* 5 to 15       */
#define DRIFT_TEMP_BIN_2_UPPER    (DRIFT_TEMP_BIN_1_LOWER)
#define DRIFT_TEMP_BIN_3_LOWER    (376)                      /* 15 to 20      */
#define DRIFT_TEMP_BIN_3_UPPER    (DRIFT_TEMP_BIN_2_LOWER)
#define DRIFT_TEMP_BIN_4_LOWER    (370)                      /* 20 to 25      */
#define DRIFT_TEMP_BIN_4_UPPER    (DRIFT_TEMP_BIN_3_LOWER)
#define DRIFT_TEMP_BIN_5_LOWER    (362)                      /* 25 to 30      */
#define DRIFT_TEMP_BIN_5_UPPER    (DRIFT_TEMP_BIN_4_LOWER)
#define DRIFT_TEMP_BIN_6_LOWER    (356)                      /* 30 to 35      */
#define DRIFT_TEMP_BIN_6_UPPER    (DRIFT_TEMP_BIN_5_LOWER)
#define DRIFT_TEMP_BIN_7_LOWER    (347)                      /* 35 to 40      */
#define DRIFT_TEMP_BIN_7_UPPER    (DRIFT_TEMP_BIN_6_LOWER)
#define DRIFT_TEMP_BIN_8_LOWER    (342)        /* Not used - placeholder      */
#define DRIFT_TEMP_BIN_8_UPPER    (DRIFT_TEMP_BIN_7_LOWER)   /* 40 to 45  */
#define DRIFT_TEMP_BIN_9_LOWER    (334)                      /* 45 to 50      */
#define DRIFT_TEMP_BIN_9_UPPER    (DRIFT_TEMP_BIN_8_LOWER)
#define DRIFT_TEMP_BIN_10_LOWER   (334)                      /* > 50          */
#define DRIFT_TEMP_BIN_10_UPPER   (DRIFT_TEMP_BIN_9_LOWER)



#define TEMP_CNT_MIN            (292)
#define TEMP_CNT_MAX            (431)

/* Custom tmd2755 configuration */
#define CUSTOM_PULSE_CNT        (31)
#define CUSTOM_PULSE_LEN        (38)
#define CUSTOM_PGAIN1            (0)
#define CUSTOM_PGAIN2            (3)
#define CUSTOM_PLDRIVE           (8)
#define CUSTOM_PRATE            (31)
#define CUSTOM_PWEN   (TMD2755_PWEN)  /* PW is enabled                 */
#define CUSTOM_PWTIME           (35)  /* (x + 1)*2.78ms -> 100 ms      */
#define CUSTOM_BIN_TGT_SRCH      (4)
#define CUSTOM_ORE               (1)
#define CUSTOM_APC_DISABLE       (PROX_APC_DISABLED)
#define CUSTOM_HW_AVG            (2)
#define CUSTOM_DET_THRESH       (70)  /* expanded to 10 bits: 280      */
#define CUSTOM_REL_THRESH       (32)  /* expanded to 10 bits: 130      */

/* pdata range for updating drift table */
#define DRIFT_TABLE_PDATA_LOWER     (10)  /* DLL - 14  */
#define DRIFT_TABLE_PDATA_UPPER     (66)  /* DUL1 + 14 */

/* pdata out of range */
#define PDATA_OUT_OF_RANGE_MAX      (2)
#define PDATA_OUT_OF_RANGE_FIRST    (1)
#define PDATA_OUT_OF_RANGE_INIT     (0)

enum tmd2755_drift_regs
{
    TMD2755_REG_TEMP_ENABLE    = 0x91,   /* enable temp sensor    */
    TMD2755_REG_DRFT_ADC_CFG   = 0xBF,   /* bit 0: 0 prox, 1 temp */
    TMD2755_REG_DRFT_CALIB     = 0xD7,   /* manual start adc      */
};

enum tmd2755_drift_mask_shift_reg
{
    TMD2755_MASK_DRIFT_ADC_SEL     = 0x3,
    TMD2755_SHIFT_DRIFT_ADC_SEL    = 0,

    TMD2755_MASK_DRIFT_ADC_SETTLE  = 0x0C,
    TMD2755_SHIFT_DRIFT_ADC_SETTLE = 2,

    TMD2755_MASK_DRIFT_MAN_ADC_START   = 0x02,
    TMD2755_SHIFT_DRIFT_MAN_ADC_START  = 1,

    TMD2755_MASK_DRIFT_CFG1_EN_TEMP   = 0x80, /* Used to enable temp sensor */
    TMD2755_SHIFT_DRIFT_CFG1_EN_TEMP  = 7,

    TMD2755_MASK_DRIFT_PDATA_L     = 0xFF,
    TMD2755_SHIFT_DRIFT_PDATA_L    = 2,

    TMD2755_MASK_DRIFT_PDATA_H     = 0xFF,
    TMD2755_SHIFT_DRIFT_PDATA_H     = 3,
};

/* Values for the bit registers */
#define TMD2755_ADC_SEL_PROX      (0)
#define TMD2755_ADC_SEL_TEMP      (1)
#define TMD2755_ADC_SETTLE_PROX   (1) /* 2us for prox */
#define TMD2755_ADC_SETTLE_TEMP   (3) /* 8us for temp */
#define TMD2755_ADC_MANUAL_START  (1)

/* Timer */
#define TIMER_PERIOD_SEC          (8)
#define SEC_TO_MILLI           (1000)
/* Delay that allows transition from prox measurements to temp measurements */
#define ADC_TEMP_DELAY_MS       (100)
#define PTIME_COMP_DELAY_MS       (5)
#define ADC_PROX_DELAY_MS        (30)
#define PWTIME_STEP_INC_US     (2780)

typedef struct _tmd2755_thermal_drift_info_t
{
    bool init;
    struct tmd2755_chip *chip;
    bool table_modified;
    struct delayed_work drift_table_update;
}tmd2755_thermal_drift_info_t;

typedef struct _drift_table_state_t
{
    bool  state;  // false release, true detect
    u16   pdata;
    s16   poffset;
    u16   temp_cnt;
    s16   poffset_table;
    u16   table_index;
}drift_table_state_t;


void ams_tmd2755_sw_drift_setup(struct tmd2755_chip *chip);
void ams_tmd2755_sw_drift_shutdown(void);

#endif /* __AMS_TMD2755_THERMAL_DRIFT_H__ */

